@@ -1,10 +1,11 @@
-from typing import TYPE_CHECKING, Iterable, List, Optional, Union
+from copy import copy
+from typing import TYPE_CHECKING, Iterable, List, Optional, Set, Union
 
 from pipegen.config import FnGetAtt, FnSub, Ref, get_ecr_arn, parse_value
 
 from .interfaces import ResourceOutput
 
-if TYPE_CHECKING:
+if TYPE_CHECKING:  # pragma: no cover
     from typing_extensions import TypedDict
 else:
     TypedDict = object
@@ -99,29 +100,29 @@ def codepipeline_role(config, codebuild_projects: List[str]) -> ResourceOutput:
     sub_config = config.get("config", {})
     permissions = [
         iam_permission(
-            S3_BUCKET_PERMISSIONS,
+            copy(S3_BUCKET_PERMISSIONS),
             [
                 parse_value(
                     "arn:aws:s3:::${BucketName}",
-                    BucketName=str(sub_config.get("s3_bucket", "")),
+                    BucketName=str(sub_config["s3_bucket"]),
                 ),
                 parse_value(
                     "arn:aws:s3:::${BucketName}/*",
-                    BucketName=str(sub_config.get("s3_bucket", "")),
+                    BucketName=str(sub_config["s3_bucket"]),
                 ),
             ],
         ),
         iam_permission(
-            KMS_KEY_PERMISSIONS,
+            copy(KMS_KEY_PERMISSIONS),
             [
                 parse_value(
                     "${KmsKeyArn}",
-                    KmsKeyArn=sub_config.get("kms_key_arn"),
+                    KmsKeyArn=sub_config["kms_key_arn"],
                 )
             ],
         ),
         iam_permission(
-            CODEPIPELINE_CODEBUILD_PERMISSIONS,
+            copy(CODEPIPELINE_CODEBUILD_PERMISSIONS),
             [
                 {"Fn::GetAtt": [codebuild_project, "Arn"]}
                 for codebuild_project in codebuild_projects
@@ -131,17 +132,24 @@ def codepipeline_role(config, codebuild_projects: List[str]) -> ResourceOutput:
 
     # Add Source perms
     codecommit_projects: List[Union[str, FnSub, FnGetAtt, Ref]] = []
+    codestar_connection_arns: Set[str] = set()
     for source in config.get("sources", []):
-        if source.get("from", "").lower() == "codecommit":
+        if source["from"] == "CodeCommit":
             codecommit_projects.append(
                 parse_value(
                     "arn:aws:codecommit:${AWS::Region}:${AWS::AccountId}:${RepositoryName}",
-                    RepositoryName=source.get("repository"),
+                    RepositoryName=source["repository"],
                 )
             )
+        elif source["from"] == "CodeStarConnection":
+            if not source.get("connection_arn"):
+                raise RuntimeError(
+                    f"Source {source['name']} uses CodeStar Connections, but does not specify a connection_arn"
+                )
+            codestar_connection_arns.add(source.get("connection_arn"))
         else:
             raise NotImplementedError(
-                f"Source type '{source.get('from', '')}' is not supported yet"
+                f"Source type '{source['from']}' is not supported yet"
             )
 
     if codecommit_projects:
@@ -157,6 +165,21 @@ def codepipeline_role(config, codebuild_projects: List[str]) -> ResourceOutput:
                 codecommit_projects,
             )
         )
+    if codestar_connection_arns:
+        permissions.append(
+            iam_permission(
+                [
+                    "codestar-connections:UseConnection",
+                ],
+                [
+                    parse_value(
+                        "${ConnectionArn}",
+                        ConnectionArn=connection_arn,
+                    )
+                    for connection_arn in sorted(codestar_connection_arns)
+                ],
+            )
+        )
 
     return ResourceOutput(
         definition={
@@ -169,35 +192,34 @@ def codepipeline_role(config, codebuild_projects: List[str]) -> ResourceOutput:
     )
 
 
-def codebuild_role(config) -> ResourceOutput:
+def codebuild_role(
+    config, log_group_logical_id: Optional[str] = None
+) -> ResourceOutput:
     """Generate a CodeBuild role + policy resources"""
     sub_config = config.get("config", {})
     permissions = []
 
     # Add CW Logs perms
     log_group = sub_config.get("codebuild", {}).get("log_group", {})
-    if log_group.get("enabled", False):
+    if log_group.get("enabled"):
+        log_group_arn: Union[str, FnSub, Ref, FnGetAtt] = parse_value(
+            "arn:aws:logs:${AWS::Region}:${AWS::AccountId}:log-group:${LogGroupName}:*",
+            LogGroupName=log_group.get("name"),
+        )
+        if log_group_logical_id:
+            log_group_arn = {"Fn::GetAtt": [log_group_logical_id, "Arn"]}
+
         permissions.append(
             iam_permission(
                 ["logs:CreateLogStream", "logs:PutLogEvents"],
-                [
-                    parse_value(
-                        "arn:aws:logs:${AWS::Region}:${AWS::AccountId}:log-group:${LogGroupName}",
-                        LogGroupName=log_group.get("name", ""),
-                    ),
-                    parse_value(
-                        "arn:aws:logs:${AWS::Region}:${AWS::AccountId}:log-group:${LogGroupName}:log-stream:*",
-                        LogGroupName=log_group.get("name", ""),
-                    ),
-                ],
+                [log_group_arn],
             )
         )
 
-    default_image = sub_config.get("codebuild", {}).get("image")
     images = set()
     for stage in config.get("stages", []):
         for action in stage.get("actions", []):
-            images.add(action.get("image", default_image))
+            images.add(action["image"])
 
     image_arns = sorted(list(get_ecr_arns(list(images))))
     if image_arns:
@@ -214,24 +236,24 @@ def codebuild_role(config) -> ResourceOutput:
     permissions.extend(
         [
             iam_permission(
-                S3_BUCKET_PERMISSIONS,
+                copy(S3_BUCKET_PERMISSIONS),
                 [
                     parse_value(
                         "arn:aws:s3:::${BucketName}",
-                        BucketName=sub_config.get("s3_bucket"),
+                        BucketName=sub_config["s3_bucket"],
                     ),
                     parse_value(
                         "arn:aws:s3:::${BucketName}/*",
-                        BucketName=sub_config.get("s3_bucket"),
+                        BucketName=sub_config["s3_bucket"],
                     ),
                 ],
             ),
             iam_permission(
-                KMS_KEY_PERMISSIONS,
+                copy(KMS_KEY_PERMISSIONS),
                 [
                     parse_value(
                         "${KmsKeyArn}",
-                        KmsKeyArn=sub_config.get("kms_key_arn"),
+                        KmsKeyArn=sub_config["kms_key_arn"],
                     )
                 ],
             ),
@@ -252,3 +274,29 @@ def codebuild_role(config) -> ResourceOutput:
         },
         logical_id="CodeBuildRole",
     )
+
+
+def cloud_watch_event_role(codepipeline_logical_id: str) -> ResourceOutput:
+    """Generate a CloudWatch event role to kick off CodePipelines"""
+    permissions = [
+        iam_permission(
+            ["codepipeline:StartPipelineExecution"],
+            [
+                {
+                    "Fn::Sub": f"arn:aws:codepipeline:${{AWS::Region}}:${{AWS::AccountId}}:${{{codepipeline_logical_id}}}"
+                }
+            ],
+        )
+    ]
+
+    return ResourceOutput(
+        definition={
+            **generate_role(
+                "CloudWatchEventsRole",
+                "events.amazonaws.com",
+                ["CloudWatchEventsPolicy"],
+            ),
+            **generate_managed_policy("CloudWatchEventsPolicy", permissions),
+        },
+        logical_id="CloudWatchEventsRole",
+    )
